//
//  ViewController.swift
//  TestBeeWi-Swift
//
//  Created by Pierre Emmanuel Aguera on 08/06/2017.
//  Copyright © 2017 Pierre Emmanuel Aguera. All rights reserved.
//

import UIKit
import CoreBluetooth




class ViewController: UIViewController, CBCentralManagerDelegate, CBPeripheralDelegate {

	enum AppState {case undefinedState, initializedState, searchingState, periphFoundState, connectedState, gettingDescState, descFoundState, readingState}

	@IBOutlet var searchBtn: UIButton?
	@IBOutlet var connectBtn: UIButton?
	@IBOutlet var descBtn: UIButton?
	@IBOutlet var readBtn: UIButton?
	@IBOutlet var disconBtn: UIButton?
	
	@IBOutlet var tempLab: UILabel?
	@IBOutlet var humidLab: UILabel?
	@IBOutlet var batLab: UILabel?
	@IBOutlet var periphLab: UILabel?

	private var centralManager: CBCentralManager?
	private var thermPeripheral: CBPeripheral?
	private var services: [CBUUID]=[]
	private var dataChar: CBCharacteristic?
	private var temperature: Float
	private var humidity: Int
	private var battery: Int
	private var periphName: String
	
	private var appState: AppState
	
	private let BEEWI_SERVICE_1_String = "A8B3FA04-4834-4051-89D0-3DE95CDDD318"
	private let BEEWI_DATA_Characteristic = "A8B3FB43-4834-4051-89D0-3DE95CDDD318"
	private let NO_PERIPH = "Pas de périphérique"
	
	
	
	
	required init?(coder aDecoder: NSCoder) {

		services.append(CBUUID(string:BEEWI_SERVICE_1_String))
		
		// Peripheral initialization
		thermPeripheral = nil;
		
		// Data characteristic initialization
		dataChar = nil;
		
		appState = .undefinedState;
		
		periphName = NO_PERIPH;
		
		temperature = 0.0
		humidity = 0
		battery = 0

		super.init(coder: aDecoder)
		
		centralManager = CBCentralManager.init(delegate:self, queue:nil)
	}
	
	
	
	override func viewDidLoad() {
		super.viewDidLoad()
		// Do any additional setup after loading the view, typically from a nib.
		
		updateInterface()
	}

	override func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning()
		// Dispose of any resources that can be recreated.
	}
	
	
	func updateInterface() {
		if ((thermPeripheral != nil) && (dataChar != nil)) {
			tempLab?.text = String(format:"Température %.1f °C", temperature)
			humidLab?.text = String(format:"Humidité %ld %%", humidity)
			batLab?.text = String(format:"Batterie %ld %%", battery)
		} else {
			tempLab?.text = "Température --.-"
			humidLab?.text = "Humidité --"
			batLab?.text = "Batterie --"
		}
		
		switch appState {
		case .undefinedState,
				 .initializedState:
			searchBtn?.isEnabled = true
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = false
			readBtn?.isEnabled = false
			disconBtn?.isEnabled = false
			
			periphName = "Pas de périphérique"
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.darkGray
		case .searchingState:
			searchBtn?.isEnabled = true
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = false
			readBtn?.isEnabled = false
			disconBtn?.isEnabled = false
			
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.darkGray
		case .periphFoundState:
			searchBtn?.isEnabled = true
			connectBtn?.isEnabled = true
			descBtn?.isEnabled = false
			readBtn?.isEnabled = false
			disconBtn?.isEnabled = false
			
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.red
		case .connectedState:
			searchBtn?.isEnabled = false
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = true
			readBtn?.isEnabled = false
			disconBtn?.isEnabled = true
			
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.green
		case .gettingDescState:
			searchBtn?.isEnabled = false
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = false
			readBtn?.isEnabled = false
			disconBtn?.isEnabled = true
			
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.green
		case .descFoundState:
			searchBtn?.isEnabled = false
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = false
			readBtn?.isEnabled = true
			disconBtn?.isEnabled = true
			
			periphLab?.text = periphName
			periphLab?.textColor = UIColor.green
		case .readingState:
			searchBtn?.isEnabled = false
			connectBtn?.isEnabled = false
			descBtn?.isEnabled = false
			readBtn?.isEnabled = true
			disconBtn?.isEnabled = true

			periphLab?.text = periphName
			periphLab?.textColor = UIColor.green
		}
		
	}


	// MARK: - CBCentralManagerDelegate
	
	func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
		print("Central manager did connect to peripheral")

		appState = .connectedState
		
		// Update interface
		updateInterface()
	}
	

	func centralManager(_ central: CBCentralManager, didFailToConnect peripheral: CBPeripheral, error: Error?) {
		print("Central manager failed to connect to peripheral")
		
		appState = .initializedState
		
		// Update interface
		updateInterface()
	}

	
	func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
		print("Central manager did disconnect from peripheral")
		
		appState = .periphFoundState
		
		// Update interface
		updateInterface()
	}

	
	func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String : Any], rssi RSSI: NSNumber) {
		let localName: String? = advertisementData[CBAdvertisementDataLocalNameKey] as! String?
		if (localName != nil) && (strlen(localName)) > 0 {
			print("Found the device: \(String(describing: localName))")
			if (localName?.compare("BeeWi SmartClim") == .orderedSame) {
				periphName = localName!
				
				// End scanning
				centralManager?.stopScan()
				
				// Retain peripheral
				thermPeripheral = peripheral
				
				appState = .periphFoundState
				
				// Update interface
				updateInterface()
			}
		}
	}
	
	
	func centralManagerDidUpdateState(_ central: CBCentralManager) {
		switch central.state {
		case .poweredOn:
			print("CoreBluetooth BLE hardware is powered on and ready")
			appState = .initializedState
		case .poweredOff:
			print("CoreBluetooth BLE hardware is powered off")
			appState = .undefinedState
		case .unauthorized:
			print("CoreBluetooth BLE state is unauthorized")
			appState = .undefinedState
		case .unknown:
			print("CoreBluetooth BLE state is unknown")
			appState = .undefinedState
		case .unsupported:
			print("CoreBluetooth BLE hardware is unsupported on this platform")
			appState = .undefinedState
		case .resetting:
			print("CoreBluetooth BLE hardware is resetting")
			appState = .undefinedState
		}
		
	// Update interface
	updateInterface()
	}


	// MARK: - CBPeripheralDelegate
	
	// Invoked when you discover the peripheral's available services.
	func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
		for service in peripheral.services! {
			print("Discovered service \(service)")
			print("Discovering characteristics for service %@", service)
			peripheral.discoverCharacteristics(nil, for: service)
			}
	}
	
	
	// Invoked when you discover the characteristics of a specified service.
	func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
		for characteristic in service.characteristics! {
			print("Discovered characteristic %@", characteristic)
			if characteristic.uuid.uuidString.compare(BEEWI_DATA_Characteristic) == .orderedSame {
				dataChar = characteristic
				
				appState = .descFoundState
				
				// Update GUI
				updateInterface()
				}
			}
	}
	
	
	// Invoked when you retrieve a specified characteristic's value, or when the peripheral device notifies your app that the characteristic's value has changed.
	func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor characteristic: CBCharacteristic, error: Error?) {
		if characteristic.uuid == dataChar?.uuid {
			let data : Data = characteristic.value!
			if data.count != 10 {
				print("bad data length \(data.count) (should be 10)")
			} else {
				/*# the temperature consists of 3 bytes
				# Posivive value: byte 1 & 2 present the tenfold of the temperature
				# Negative value: byte 2 - byte 3 present the tenfold of the temperature */
				let data_buf = [UInt8](data)
				var temperature_buf = [UInt8](repeating: 0, count: 3)
				temperature_buf[0] = data_buf[0] & 0x000000ff
				temperature_buf[1] = data_buf[1] & 0x000000ff
				temperature_buf[2] = data_buf[2] & 0x000000ff
				let temperatureInt : Int = (Int(temperature_buf[2])*255)+Int(temperature_buf[1])
				temperature = Float(temperatureInt) / 10.0
				
				let humidity_buf: UInt8 = data_buf[4]
				humidity = Int(humidity_buf)
				
				let battery_buf : UInt8 = data_buf[9]
				battery = Int(battery_buf)
				
				appState = .descFoundState
				
				// Update GUI (labels)
				updateInterface()
			}
		}
	}

	// MARK: - Actions
	@IBAction func searchPeripheral(sender: UIButton) {
		// Doesn't advertise for service
		centralManager?.scanForPeripherals(withServices: nil,  options:nil)
	
		appState = .searchingState
		// Update interface
		updateInterface()
	}
	
	
	@IBAction func connectToPeripheral(sender: UIButton) {
		// Connect
		thermPeripheral?.delegate = self
		
		centralManager?.connect(thermPeripheral!, options:nil)
	}

	
	@IBAction func readDescFromPeripheral(sender: UIButton) {
		// Read service and characteristic description
		thermPeripheral?.discoverServices(services)
		
		appState = .gettingDescState
		// Update interface
		updateInterface()
	}
	
	
	@IBAction func readData(sender: UIButton) {
		if ((thermPeripheral != nil) && (dataChar != nil)) {
			appState = .readingState
			// Update interface
			updateInterface()

			thermPeripheral?.readValue(for: dataChar!)
		}
	}
	
	
	@IBAction func disconnectFromPeripheral(sender: UIButton) {
		// Disconnect
		centralManager?.cancelPeripheralConnection(thermPeripheral!)
		dataChar = nil
	}
	
	
}
